.import "../ui/utils.js" as Utils

function getWebsiteNewEventUrl(date) {
    function dateString(year, month, day) {
        var s = '' + year;
        s += (month < 10 ? '0' : '') + month;
        s += (day < 10 ? '0' : '') + day;
        return s;
    }

    var nextDay = new Date(date.getFullYear(), date.getMonth(), date.getDate() + 1)

    var url = 'https://calendar.google.com/calendar/render?action=TEMPLATE'
    var startDate = dateString(date.getFullYear(), date.getMonth() + 1, date.getDate())
    var endDate = dateString(nextDay.getFullYear(), nextDay.getMonth() + 1, nextDay.getDate())
    url += '&dates=' + startDate + '/' + endDate
    return url
}

function quickCreateEvent(args, callback) {
    https://www.googleapis.com/calendar/v3/calendars/calendarId/events/quickAdd
    var text = args.text;
    if (args.date) {
        var dateString = args.date.getFullYear() + '-' + (args.date.getMonth() + 1) + '-' + args.date.getDate();
        text = dateString + ' ' + text;
    }
    var url = 'https://www.googleapis.com/calendar/v3';
    url += '/calendars/'
    url += encodeURIComponent(args.calendarId);
    url += '/events/quickAdd';
    url += '?text=' + encodeURIComponent(text);
    Utils.postJSON({
        url: url,
        headers: {
            "Authorization": "Bearer " + args.access_token,
        }
    }, function(err, data, xhr) {
        console.log('quickCreateEvent.response', err, data, xhr.status);
        if (!err && data && data.error) {
            return callback(data, null, xhr);
        }
        callback(err, data, xhr);
    });
}

function fetchCalendarList(args, callback) {
    var url = 'https://www.googleapis.com/calendar/v3/users/me/calendarList';
    Utils.getJSON({
        url: url,
        headers: {
            "Authorization": "Bearer " + args.access_token,
        }
    }, function(err, data, xhr) {
        console.log('fetchCalendarList.response', err, data, xhr.status);
        if (!err && data && data.error) {
            return callback(data, null, xhr);
        }
        callback(err, data, xhr);
    });
}

function pollAccessToken(args, callback) {
    var url = 'https://www.googleapis.com/oauth2/v4/token';
    Utils.post({
        url: url,
        data: {
            client_id: args.client_id,
            client_secret: args.client_secret,
            code: args.device_code,
            grant_type: 'http://oauth.net/grant_type/device/1.0',
        },
    }, function(err, data, xhr) {
        console.log('test', err, data)
        data = JSON.parse(data);
        console.log('data', JSON.stringify(data, null, '\t'));

        if (data.error) {
            // Not yet ready
            return callback(data.error, data, xhr);
        }

        callback(null, data, xhr);
    });
}

function fetchUserCode(args, callback) {
    var url = 'https://accounts.google.com/o/oauth2/device/code';
    Utils.post({
        url: url,
        data: {
            client_id: args.client_id,
            scope: 'https://www.googleapis.com/auth/calendar',
        },
    }, callback);
}

function fetchNewAccessToken(args, callback) {
    var url = 'https://www.googleapis.com/oauth2/v4/token';
    Utils.post({
        url: url,
        data: {
            client_id: args.client_id,
            client_secret: args.client_secret,
            refresh_token: args.refresh_token,
            grant_type: 'refresh_token',
        },
    }, callback);
}


var QmlGoogleSession = function(client_id, client_secret, timer) {
    this.client_id = client_id;
    this.client_secret = client_secret;
    this.timer = timer;
    this.device_code = null;
    this.access_token = null;
    this.access_token_expires_at = 0;
    this.refresh_token = null;
    return this;
};

// QmlGoogleSession.prototype.checkResponse = function(callback) {
//     return function(err, data, xhr) {
//         if (!err && data && data.error) {
//             return callback(data, null, xhr);
//         }
//         return callback(err, data, xhr);
//     };
// };

QmlGoogleSession.prototype.refresh = function(callback) {
    // @param callback: function() Errors are handled silently.
    console.log('QmlGoogleSession.refresh')
    var _this = this;
    fetchNewAccessToken({
        client_id: this.client_id,
        client_secret: this.client_secret,
        refresh_token: this.refresh_token,
    }, function(err, data, xhr) {
        console.log('QmlGoogleSession.refresh.onData')
        if (err) {
            return console.log('Error when using refreshToken:', err, data);
        }
        data = JSON.parse(data);
        // console.log('QmlGoogleSession.refresh.response', JSON.stringify(data, null, '\t'));

        var access_token_expires_at = Date.now() + data.expires_in * 1000;

        console.log('access_token', _this.access_token + ' => ' + data.access_token);
        console.log('access_token_expires_at', _this.access_token_expires_at + ' => ' + access_token_expires_at);

        _this.access_token = data.access_token;
        _this.access_token_type = data.token_type;
        _this.access_token_expires_at = access_token_expires_at;

        _this.onNewAccessToken();
        callback();
    })
};

QmlGoogleSession.prototype.onNewAccessToken = function() {
    console.log('QmlGoogleSession.onNewAccessToken')
    // to be overloaded
};


QmlGoogleSession.prototype.refreshAndCall = function(func) {
    console.log('QmlGoogleSession.refreshAndCall')
    // Check if access_token has expired (60 second threshold).
    if (Date.now() >= (this.access_token_expires_at - 60000)) {
        this.refresh(func);
    } else {
        func();
    }
};


QmlGoogleSession.prototype.fetchCalendarList = function(args, callback) {
    console.log('QmlGoogleSession.fetchCalendarList')
    args.access_token = this.access_token;
    var func = fetchCalendarList.bind(null, args, callback);
    this.refreshAndCall(func);
};



QmlGoogleSession.prototype.quickCreateEvent = function(args, callback) {
    console.log('QmlGoogleSession.quickCreateEvent')
    args.access_token = this.access_token;
    var func = quickCreateEvent.bind(null, args, callback);
    this.refreshAndCall(func);
};

